@@ -995,9 +995,11 @@ extern "C" {
         _terms.reset();
         _guards.reset();
         for (solver::solution const& s : solutions) {
+            if (!s.term)
+                continue;
             _vars.push_back(s.var);
             _terms.push_back(s.term);
-            _guards.push_back(s.guard);
+            _guards.push_back(s.guard ? s.guard : m.mk_true());
         }
         Z3_CATCH;
     }
@@ -1185,6 +1187,7 @@ extern "C" {
         Z3_TRY;
         LOG_Z3_solver_set_initial_value(c, s, var, value);
         RESET_ERROR_CODE();
+        init_solver(c, s);
         if (to_expr(var)->get_sort() != to_expr(value)->get_sort()) {
             SET_ERROR_CODE(Z3_INVALID_USAGE, "variable and value should have same sort");
             return;
