@@ -93,7 +93,7 @@ namespace euf {
             return;
         for (auto arg : enode_args(n))
             if (is_op(arg))
-                register_shared(arg); // TODO optimization to avoid registering shared terms twice
+                register_shared(arg); 
     }
 
     void ac_plugin::register_shared(enode* n) {
@@ -109,7 +109,7 @@ namespace euf {
         m_shared_nodes.setx(n->get_id(), true, false);
         sort(monomial(m));
         m_shared_todo.insert(m_shared.size());
-        m_shared.push_back({ n, m, justification::axiom() });
+        m_shared.push_back({ n, m, justification::axiom(get_id()) });
         push_undo(is_register_shared);
     }
 
@@ -180,7 +180,7 @@ namespace euf {
     std::ostream& ac_plugin::display_monomial(std::ostream& out, ptr_vector<node> const& m) const {
         for (auto n : m) {
             if (n->n->num_args() == 0)
-                out << mk_pp(n->n->get_expr(), g.get_manager()) << " ";
+                out << n->n->get_expr_id() << ": " << mk_pp(n->n->get_expr(), g.get_manager()) << " ";
             else
                 out << g.bpp(n->n) << " ";
         }
@@ -244,6 +244,7 @@ namespace euf {
         if (l == r)
             return;
         auto j = justification::equality(l, r);
+        TRACE(plugin, tout << g.bpp(l) << " == " << g.bpp(r) << " " << is_op(l) << " " << is_op(r) << "\n");
         if (!is_op(l) && !is_op(r))
             merge(mk_node(l), mk_node(r), j);
         else
@@ -263,6 +264,7 @@ namespace euf {
     void ac_plugin::init_equation(eq const& e) {
         m_eqs.push_back(e);
         auto& eq = m_eqs.back();
+        TRACE(plugin, display_equation(tout, e) << "\n");
         if (orient_equation(eq)) {
             
             unsigned eq_id = m_eqs.size() - 1;
@@ -273,6 +275,8 @@ namespace euf {
                     n->root->n->mark1();
                     push_undo(is_add_eq_index);
                     m_node_trail.push_back(n->root);
+                    for (auto s : n->root->shared)
+                        m_shared_todo.insert(s);
                 }
             }
 
@@ -282,6 +286,8 @@ namespace euf {
                     n->root->n->mark1();
                     push_undo(is_add_eq_index);
                     m_node_trail.push_back(n->root);
+                    for (auto s : n->root->shared)
+                        m_shared_todo.insert(s);
                 }
             }
 
@@ -291,6 +297,7 @@ namespace euf {
             for (auto n : monomial(eq.r))
                 n->root->n->unmark1();
 
+            TRACE(plugin, display_equation(tout, e) << "\n");
             m_to_simplify_todo.insert(eq_id);
         }
         else
@@ -368,6 +375,7 @@ namespace euf {
     }
 
     void ac_plugin::merge(node* root, node* other, justification j) {
+        TRACE(plugin, tout << root << " == " << other << " num shared " << other->shared.size() << "\n");
         for (auto n : equiv(other))
             n->root = root;
         m_merge_trail.push_back({ other, root->shared.size(), root->eqs.size() });
@@ -394,22 +402,34 @@ namespace euf {
         ptr_vector<node> m;
         ns.push_back(n);
         for (unsigned i = 0; i < ns.size(); ++i) {
-            n = ns[i];
-            if (is_op(n)) 
-                ns.append(n->num_args(), n->args());            
+            auto k = ns[i];
+            if (is_op(k)) 
+                ns.append(k->num_args(), k->args());            
             else 
-                m.push_back(mk_node(n));            
+                m.push_back(mk_node(k));            
         }
         return to_monomial(n, m);
     }
 
     unsigned ac_plugin::to_monomial(enode* e, ptr_vector<node> const& ms) {
         unsigned id = m_monomials.size();
-        m_monomials.push_back({ ms, bloom() });
+        m_monomials.push_back({ ms, bloom(), e });
         push_undo(is_add_monomial);
         return id;
     }
 
+    enode* ac_plugin::from_monomial(ptr_vector<node> const& mon) {
+        auto& m = g.get_manager();
+        ptr_buffer<expr> args;
+        enode_vector nodes;
+        for (auto arg : mon) {
+            nodes.push_back(arg->root->n);
+            args.push_back(arg->root->n->get_expr());
+        }
+        auto n = m.mk_app(m_fid, m_op, args.size(), args.data());
+        return g.mk(n, 0, nodes.size(), nodes.data());
+    }
+
     ac_plugin::node* ac_plugin::node::mk(region& r, enode* n) {
         auto* mem = r.allocate(sizeof(node));
         node* res = new (mem) node();
@@ -427,6 +447,9 @@ namespace euf {
         push_undo(is_add_node);
         m_nodes.setx(id, r, nullptr);
         m_node_trail.push_back(r);
+        if (is_op(n)) {
+            // extract shared sub-expressions
+        }
         return r;
     }
 
@@ -437,11 +460,12 @@ namespace euf {
             if (eq_id == UINT_MAX)
                 break;
 
-            TRACE("plugin", tout << "propagate " << eq_id << ": " << eq_pp(*this, m_eqs[eq_id]) << "\n");
+            TRACE(plugin, tout << "propagate " << eq_id << ": " << eq_pp(*this, m_eqs[eq_id]) << "\n");
 
             // simplify eq using processed
-            for (auto other_eq : backward_iterator(eq_id))
-                TRACE("plugin", tout << "backward iterator " << eq_id << " vs " << other_eq << " " << is_processed(other_eq) << "\n");
+            TRACE(plugin, 
+                  for (auto other_eq : backward_iterator(eq_id))
+                      tout << "backward iterator " << eq_id << " vs " << other_eq << " " << is_processed(other_eq) << "\n");
             for (auto other_eq : backward_iterator(eq_id))
                 if (is_processed(other_eq) && backward_simplify(eq_id, other_eq))
                     goto loop_start;
@@ -465,7 +489,7 @@ namespace euf {
         }
         propagate_shared();
 
-        CTRACE("plugin", !m_shared.empty() || !m_eqs.empty(), display(tout));
+        CTRACE(plugin, !m_shared.empty() || !m_eqs.empty(), display(tout));
     }
 
     unsigned ac_plugin::pick_next_eq() {
@@ -636,11 +660,11 @@ namespace euf {
         auto& src = m_eqs[src_eq];  // src_r_counts, src_l_counts are initialized
         auto& dst = m_eqs[dst_eq];
 
-        TRACE("plugin", tout << "forward simplify " << eq_pp(*this, src) << " " << eq_pp(*this, dst) << "\n");
+        TRACE(plugin, tout << "forward simplify " << eq_pp(*this, src) << " " << eq_pp(*this, dst) << "\n");
 
 
         if (forward_subsumes(src_eq, dst_eq)) {
-            TRACE("plugin", tout << "forward subsumed\n");
+            TRACE(plugin, tout << "forward subsumed\n");
             set_status(dst_eq, eq_status::is_dead);
             return;
         }
@@ -690,7 +714,7 @@ namespace euf {
         push_undo(is_update_eq);
         m_src_r.reset();
         m_src_r.append(monomial(src.r).m_nodes);
-        TRACE("plugin", tout << "rewritten to " << m_pp(*this, monomial(new_r)) << "\n");
+        TRACE(plugin, tout << "rewritten to " << m_pp(*this, monomial(new_r)) << "\n");
     }
 
     bool ac_plugin::backward_simplify(unsigned dst_eq, unsigned src_eq) {
@@ -702,10 +726,10 @@ namespace euf {
         //
         // dst_ids, dst_count contain rhs of dst_eq
         //
-        TRACE("plugin", tout << "backward simplify " << eq_pp(*this, src) << " " << eq_pp(*this, dst) << " can-be-subset: " << can_be_subset(monomial(src.l), monomial(dst.r)) << "\n");
+        TRACE(plugin, tout << "backward simplify " << eq_pp(*this, src) << " " << eq_pp(*this, dst) << " can-be-subset: " << can_be_subset(monomial(src.l), monomial(dst.r)) << "\n");
 
         if (backward_subsumes(src_eq, dst_eq)) {
-            TRACE("plugin", tout << "backward subsumed\n");
+            TRACE(plugin, tout << "backward subsumed\n");
             set_status(dst_eq, eq_status::is_dead);
             return true;
         }
@@ -713,7 +737,7 @@ namespace euf {
         if (!can_be_subset(monomial(src.l), monomial(dst.r)))
             return false;
         if (!is_subset(m_dst_r_counts, m_src_l_counts, monomial(src.l))) {
-            TRACE("plugin", tout << "not subset\n");
+            TRACE(plugin, tout << "not subset\n");
             return false;
         }
 
@@ -730,7 +754,7 @@ namespace euf {
         m_update_eq_trail.push_back({ dst_eq, m_eqs[dst_eq] });
         m_eqs[dst_eq].r = new_r;
         m_eqs[dst_eq].j = j;
-        TRACE("plugin", tout << "rewritten to " << m_pp(*this, monomial(new_r)) << "\n");
+        TRACE(plugin, tout << "rewritten to " << m_pp(*this, monomial(new_r)) << "\n");
         push_undo(is_update_eq);
         return true;
     }
@@ -849,7 +873,7 @@ namespace euf {
                         continue;
                     if (!is_subset(m_m_counts, m_eq_counts, monomial(src.l)))
                         continue;
-                    TRACE("plugin", display_equation(tout << "reduce ", src) << "\n");
+                    TRACE(plugin, display_equation(tout << "reduce ", src) << "\n");
                     SASSERT(is_correct_ref_count(monomial(src.l), m_eq_counts));
                     rewrite1(m_eq_counts, monomial(src.r), m_m_counts, m);
                     j = join(j, eq);
@@ -899,7 +923,7 @@ namespace euf {
         auto& src = m_eqs[src_eq];
         auto& dst = m_eqs[dst_eq];
 
-        TRACE("plugin", tout << "superpose: "; display_equation(tout, src); tout << " "; display_equation(tout, dst); tout << "\n";);
+        TRACE(plugin, tout << "superpose: "; display_equation(tout, src); tout << " "; display_equation(tout, dst); tout << "\n";);
         // AB -> C, AD -> E => BE ~ CD
         // m_src_ids, m_src_counts contains information about src (call it AD -> E)
         m_dst_l_counts.reset();
@@ -907,7 +931,6 @@ namespace euf {
         m_dst_r.reset();
         m_dst_r.append(monomial(dst.r).m_nodes);
         unsigned src_r_size = m_src_r.size();
-        unsigned dst_r_size = m_dst_r.size();
         SASSERT(src_r_size == monomial(src.r).size());
         // dst_r contains C
         // src_r contains E
@@ -941,7 +964,7 @@ namespace euf {
             return;
         }
 
-        TRACE("plugin", tout << m_pp(*this, m_src_r) << "== " << m_pp(*this, m_dst_r) << "\n";);
+        TRACE(plugin, tout << m_pp(*this, m_src_r) << "== " << m_pp(*this, m_dst_r) << "\n";);
 
         justification j = justify_rewrite(src_eq, dst_eq);
         reduce(m_dst_r, j);
@@ -983,6 +1006,7 @@ namespace euf {
     // 
 
     void ac_plugin::propagate_shared() {
+        TRACE(plugin, tout << "num shared todo " << m_shared_todo.size() << "\n");
         if (m_shared_todo.empty())
             return;
         while (!m_shared_todo.empty()) {
@@ -994,11 +1018,11 @@ namespace euf {
         m_monomial_table.reset();
         for (auto const& s1 : m_shared) {
             shared s2;
-            TRACE("plugin", tout << "shared " << m_pp(*this, monomial(s1.m)) << "\n");
+            TRACE(plugin, tout << "shared " << m_pp(*this, monomial(s1.m)) << "\n");
             if (!m_monomial_table.find(s1.m, s2)) 
                 m_monomial_table.insert(s1.m, s1);
             else if (s2.n->get_root() != s1.n->get_root()) {
-                TRACE("plugin", tout << m_pp(*this, monomial(s1.m)) << " == " << m_pp(*this, monomial(s2.m)) << "\n");
+                TRACE(plugin, tout << m_pp(*this, monomial(s1.m)) << " == " << m_pp(*this, monomial(s2.m)) << "\n");
                 push_merge(s1.n, s2.n, justification::dependent(m_dep_manager.mk_join(m_dep_manager.mk_leaf(s1.j), m_dep_manager.mk_leaf(s2.j))));
             }
         }
@@ -1007,12 +1031,15 @@ namespace euf {
     void ac_plugin::simplify_shared(unsigned idx, shared s) {
         auto j = s.j;
         auto old_m = s.m;
+        auto old_n = monomial(old_m).m_src;
         ptr_vector<node> m1(monomial(old_m).m_nodes);
-        TRACE("plugin", tout << "simplify " << m_pp(*this, monomial(old_m)) << "\n");
+        TRACE(plugin, tout << "simplify " << g.bpp(old_n) << ": " << m_pp(*this, monomial(old_m)) << "\n");
         if (!reduce(m1, j))
             return;
 
-        auto new_m = to_monomial(m1);
+
+        auto new_n = from_monomial(m1);
+        auto new_m = to_monomial(new_n, m1);
         // update shared occurrences for members of the new monomial that are not already in the old monomial.
         for (auto n : monomial(old_m))
             n->root->n->mark1();
@@ -1029,6 +1056,10 @@ namespace euf {
         push_undo(is_update_shared);
         m_shared[idx].m = new_m;
         m_shared[idx].j = j;
+
+        TRACE(plugin, tout << "shared simplified to " << m_pp(*this, monomial(new_m)) << "\n");
+        
+        push_merge(old_n, new_n, j);
     }
 
     justification ac_plugin::justify_rewrite(unsigned eq1, unsigned eq2) {
