@@ -47,7 +47,6 @@ namespace euf {
             unsigned_vector eqs;       // equality occurrences
             
             unsigned root_id() const { return root->n->get_id(); }
-            ~node() {}
             static node* mk(region& r, enode* n);
         };
 
@@ -62,8 +61,7 @@ namespace euf {
                 node* operator*() { return m_first; }
                 iterator& operator++() { if (!m_last) m_last = m_first; m_first = m_first->next; return *this; }
                 iterator operator++(int) { iterator tmp = *this; ++*this; return tmp; }
-                bool operator==(iterator const& other) const { return m_last == other.m_last && m_first == other.m_first; }
-                bool operator!=(iterator const& other) const { return !(*this == other); }
+                bool operator!=(iterator const& other) const { return m_last != other.m_last || m_first != other.m_first; }
             };
             equiv(node& _n) :n(_n) {}
             equiv(node* _n) :n(*_n) {}
@@ -99,6 +97,7 @@ namespace euf {
         struct monomial_t {
             ptr_vector<node> m_nodes;
             bloom m_bloom;
+            enode* m_src = nullptr;
             node* operator[](unsigned i) const { return m_nodes[i]; }
             unsigned size() const { return m_nodes.size(); }
             void set(ptr_vector<node> const& ns) { m_nodes.reset(); m_nodes.append(ns); m_bloom.m_tick = 0; }
@@ -138,7 +137,7 @@ namespace euf {
         };
 
         theory_id                m_fid = 0;
-        unsigned                 m_op = null_decl_kind;
+        decl_kind                m_op = null_decl_kind;
         func_decl*               m_decl = nullptr;
         vector<eq>               m_eqs;
         ptr_vector<node>         m_nodes;
@@ -189,6 +188,7 @@ namespace euf {
         unsigned to_monomial(enode* n);
         unsigned to_monomial(enode* n, ptr_vector<node> const& ms);
         unsigned to_monomial(ptr_vector<node> const& ms) { return to_monomial(nullptr, ms); }
+        enode* from_monomial(ptr_vector<node> const& m);
         monomial_t const& monomial(unsigned i) const { return m_monomials[i]; }
         monomial_t& monomial(unsigned i) { return m_monomials[i]; }
         void sort(monomial_t& monomial);
@@ -270,8 +270,6 @@ namespace euf {
         ac_plugin(egraph& g, unsigned fid, unsigned op);
 
         ac_plugin(egraph& g, func_decl* f);
-
-        ~ac_plugin() override {}
         
         theory_id get_id() const override { return m_fid; }
 
