@@ -594,6 +594,7 @@ public:
         info get_info(expr* r) const;
         std::string to_str(expr* r) const;
         std::string to_strh(expr* r) const;
+        bool is_ground(expr* r) const { return get_info(r).interpreted; }
 
         expr_ref mk_ite_simplify(expr* c, expr* t, expr* e)
         {
@@ -635,6 +636,7 @@ public:
     }
 
     family_id get_family_id() const { return m_fid; }
+    family_id get_char_family_id() const { return ch.get_family_id(); }
 };
 
 inline std::ostream& operator<<(std::ostream& out, seq_util::rex::pp const & p) { return p.display(out); }
