@@ -63,8 +63,8 @@ namespace smt {
     class model_generator;
     class context;
 
-    struct cancel_exception : public std::exception {
-        char const * what() const noexcept override { return "smt-canceled"; }
+    struct oom_exception : public z3_error {
+        oom_exception() : z3_error(ERR_MEMOUT) {}
     };
 
     struct enode_pp {
@@ -1201,7 +1201,7 @@ namespace smt {
             if (act > ACTIVITY_LIMIT)
                 rescale_bool_var_activity();
             m_case_split_queue->activity_increased_eh(v);
-            TRACE("case_split", tout << "v" << v << " " << m_bvar_inc << " -> " << act << "\n";);
+            TRACE(case_split, tout << "v" << v << " " << m_bvar_inc << " -> " << act << "\n";);
         }
 
     protected:
